@@ -1,5 +1,4 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
+from __future__ import absolute_import, unicode_literals
 
 from oauthlib.common import generate_client_id as oauthlib_generate_client_id
 from oauthlib.common import UNICODE_ASCII_CHARACTER_SET
