@@ -11,6 +11,7 @@ from django.utils import timezone
 from oauth2_provider.models import get_access_token_model, get_application_model
 from oauth2_provider.settings import oauth2_settings
 
+
 Application = get_application_model()
 AccessToken = get_access_token_model()
 UserModel = get_user_model()
@@ -37,7 +38,7 @@ class TestTokenIntrospectionViews(TestCase):
             user=self.resource_server_user, token="12345678900",
             application=self.application,
             expires=timezone.now() + datetime.timedelta(days=1),
-            scope="read write introspection"
+            scope="introspection"
         )
 
         self.valid_token = AccessToken.objects.create(
